// The current plq.sci file loads the functions in the CCA toolbox to manipulate
// Piecewise Linear-Quadratic functions.  There are PLQ algorithms for computing
// all of the major transforms implemented by this toolbox. See the help files
// for more information on the functions.

function version=plq_version() 
    version = "1.5.4";

    // 1.5.3 -> 1.5.4
    // - Made plq_clean remove redundant infinities.
    // - help("plq_function") is now a central place for PLQ documentation.
    // - Added help files for plq_co and plq_conv_on_interval.
    // - Renamed qplot to plq_plot.
    // - Wrote plotting functions plq_plot2, plq_plotpa_{lambda,mu}Set.
    // - Lots of small bugs fixed, e.g.
    //   - plq_build_d didn't work with %infs correctly.
    //   - plq_clean now removes redundant infinities.

    //1.5.1 -> 1.5.2
    //added test in plq_me for negative lambda (removed)
    //added test in plq_add for adding indicator
    //update demos/llnonconvexabs.sci
    //added parameter isConvex to plq_lft, plq_pa, plq_me
    //added from mike/research/cca(2007-2008)/plq.sci on 2007-09-23
    //      - plq_co 
    //      - remove infinity values in plq_clean
    //added plq_clean at the end of plq_build
    //added new function plq_plotpa

    //1.5.0 -> 1.5.1
    //created plq_build_d and modified plq_build
    //modified plq_clean to round to 0 small numbers
    //added plq_proj and test_plq_proj
endfunction

function [plq,d] = plq_proj(f,x,p)
//projection in norm p (p in 1, 2, +%inf) of f (nonconvex) using grid x
//plq is a clean up of the resulting piecewise affine function
//d is the distance between the 2 functions d=||f-u||_p
//use only pointwise evaluation of the function to compute pointwise evaluation
//of the projection using quadratic programming

  if argn(2)<=2, p=2;end;//default to L2 norm
  if p == 2 then
    n=length(x);
    h=x(2)-x(1);//grid stepsize (assume uniform grid)
    //min||f-u||_p st u convex
    Q=2*eye(n,n)*h;p=-2*f*h;//minimize 0.5*u'*Q*u + p'*u
    //u convex means u_i+2 + u_i >= 2 u_i+1 for a regular grid.
    o=ones(n,1);  C=diag(o)-2*diag(o(1:$-1),1)+diag(o(1:$-2),2);
    C=C(1:$-2,:);//remove last 2 rows
    C=-C;//need C x <= b
    b=zeros(n-2,1);
    [u,lagr,val]=quapro(Q,p,C,b);//use quapro since qld doesnot seem reliable as of Scilab 4.1 // ci=[];cs=[];me=0;[u,lagr,info]=qld(Q,p,C,b,ci,cs,me);
    d=0.5*u'*Q*u+p'*u+f'*f*h;
    plq=plq_build_d(x,u);
    plq=plq_clean(plq);
  else
    plq = []; d = 0;
    cerror(sprintf("plq_proj: option p=%i not implemented", d));
  end
endfunction


function [plqm] = plq_min(plqf1,plqf2),
    plqm = plq_scalar(plq_max(plq_scalar(plqf1,-1),plq_scalar(plqf2,-1)),-1);
endfunction

function [plqm] = plq_max(plqf1,plqf2)
  plqm = plq_oploop(plqf1,plqf2,_plq_max_indicator,_plq_maxoninterval);
endfunction

function [plqa] = plq_add(plqf1,plqf2)
  plqa = plq_oploop(plqf1,plqf2,_plq_add_indicator,_plq_addoninterval);
endfunction

function [plqo] = plq_oploop(plqf1,plqf2,_plq_op_indicator,_plq_oponinterval)
  //initialization
    f1row = 1;f2row = 1;
    plqo=[];
    
    //sorting the domain partitions of the plq functions
  f1x=plqf1(:,1);
    f2x=plqf2(:,1);
    z = unique(gsort([f1x;f2x],'c','i'));
    p = size(z,1);
  if (size(plqf1,1) == 1 & plqf1(1,1) < %inf) | (size(plqf2,1) == 1 & plqf2(1,1) < %inf) then //plqf1 is singleton indicator function.
        plqo = _plq_op_indicator(plqf1,plqf2);
  else //plqf1,plqf2 are neither singleton indicator functions.
        plqo =[plqo;_plq_oponinterval(plqf1(1,2:$),plqf2(1,2:$),-%inf,z(1));];
        //Main Loop
          for i=1:(p-1),
              //for plq's that have common partition point --See test_plq_add "testQLQmoothPlusQLQNonsmooth"
              if plqf1(f1row,1) == z(i) & plqf2(f2row,1) == z(i) then
                  f1row = f1row + 1;
                  f2row = f2row + 1;
              elseif plqf1(f1row,1) == z(i) then f1row = f1row + 1;
                    elseif plqf2(f2row,1) == z(i) then f2row = f2row + 1;
                    else printf("\n Problem (x(i) must be increasing)");
                    end;
                    plqo =[plqo;_plq_oponinterval(plqf1(f1row,2:$),plqf2(f2row,2:$),z(i),z(i+1));]
                end;
  end;  
  plqo = plq_clean(plqo);
endfunction

function [plqa] = _plq_addoninterval(q1,q2,lb,ub) 
 plqa = [ub,(q1+q2)];
 //printf("\n Add: q1=[%i,%i,%i],q2=[%i,%i,%i] lb=%i,ub=%i -- plqa=[%i,%i,%i,%i]",q1(1,1),q1(1,2),q1(1,3),q2(1,1),q2(1,2),q2(1,3),lb,ub,plqa(1,1),plqa(1,2),plqa(1,3),plqa(1,4));
endfunction

function [plqmi] = _plq_max_indicator(plqf1,plqf2)
  if (size(plqf1,1) == 1 & plqf1(1,1) < %inf) & (size(plqf2,1) == 1 & plqf2(1,1) < %inf) then
      //both plqf1,plqf2 are singleton indicator functions
      if plqf1(1,1) ~= plqf2(1,1) then plqmi = [%inf,0,0,%inf];
      else //plqf1(1,1) == plqf2(1,2) have a singleton point in common
          y1 = plq_eval(plqf1,plqf1(1,1));
          y2 = plq_eval(plqf2,plqf2(1,1));
          if y1 >= y2 then plqmi=plqf1; else plqmi=plqf2; end; 
      end;
  elseif (size(plqf1,1) == 1 & plqf1(1,1) < %inf) then //plqf1 is a singleton indicator function
      y1 = plq_eval(plqf1,plqf1(1,1));
      y2 = plq_eval(plqf2,plqf1(1,1));
      if y1 >= y2 then plqmi=plqf1; else plqmi=[plqf1(1,1),0,0,y2]; end;
  else //plqf2 is a singleton indicator function
      y1 = plq_eval(plqf2,plqf2(1,1));
      y2 = plq_eval(plqf1,plqf2(1,1));
      if y1 >= y2 then plqmi=plqf2; else plqmi=[plqf2(1,1),0,0,y2]; end;
  end
endfunction

function [plqai] = _plq_add_indicator(plqf1,plqf2)
  if (size(plqf1,1) == 1 & plqf1(1,1) < %inf) & (size(plqf2,1) == 1 & plqf2(1,1) < %inf) then
      //both plqf1,plqf2 are singleton indicator functions
      if plqf1(1,1) ~= plqf2(1,1) then plqai = [%inf,0,0,%inf];
      else plqai = [plqf1(1,1),0,0,plqf1(1,4)+plqf2(1,4)]
      end;
  elseif (size(plqf1,1) == 1 & plqf1(1,1) < %inf) then //plqf1 is a singleton indicator function
      y2 = plq_eval(plqf2,plqf1(1,1));
      plqai = [plqf1(1,1),0,0,plqf1(1,4)+y2];
  else //plqf2 is a singleton indicator function
      y2 = plq_eval(plqf1,plqf2(1,1));
      plqai = [plqf2(1,1),0,0,plqf2(1,4)+y2];
  end
endfunction



//l1,l2,q1,q2 are all represented by the formar [a,b,c] which represents ax^2 + bx + c
//Each corresponding function will find the maximum function between two functions
//either quadratic-linear, quadratic-quadratic, linear-linear which is valid only 
//within the interval [lb,ub] the lower and upper bound.
//main function 
//computes the maximum of two functions under the assumption that q1 - q2 is linear or quadratic.
//DOES NOT HANDLE SINGLETON INDICATOR FUNCTIONS.
function plq = _plq_maxoninterval(q1,q2,lb,ub)
    //if and(q1==q2) then plq=[ub,0,q1]; return; end; //degenerate case
    //printf("\n q1=[%i,%i,%i],q2=[%i,%i,%i] lb=%i,ub=%i",q1(1,1),q1(1,2),q1(1,3),q2(1,1),q2(1,2),q2(1,3),lb,ub);
    if and(q1==q2) then
      plq=[ub,q1]; //printf(" plq_max: [%i,%i,%i,%i]",plq(1,1),plq(1,2),plq(1,3),plq(1,4)); 
      return;
    end;
    //if lb>=ub then plq=[];return;end;//degenerate case
    //to find where q1 > q2 find where (q1 - q2) > 0    
    dlq = (q1 - q2); //WARNING: the difference of two quadratics is linear! or quadratic!
    a=dlq(1,1);b=dlq(1,2);c=dlq(1,3);
    if a == 0 then
      //dlq is linear
      plq = dl_maxoninterval(q1,q2,lb,ub,a,b,c); //printf(" plq_max: [%i,%i,%i,%i]",plq(1,1),plq(1,2),plq(1,3),plq(1,4)); 
    else
      //dlq is quadratic
      plq = dq_maxoninterval(q1,q2,lb,ub,a,b,c); //printf(" plq_max: [%i,%i,%i,%i] Q",plq(1,1),plq(1,2),plq(1,3),plq(1,4)); 
    end
endfunction

//computes the maximum of two functions under the assumption that difference of q1 - q2 is linear function!!!!!
function plq = dl_maxoninterval(q1,q2,lb,ub,a,b,c)
    if b == 0 then //horizontal line, i.e. no intersection with the x-axis
        plq = maxoninterval(q1,q2,lb,ub,a,b,c);
    else 
        x(1) = -c/b; //y=bx+c --> find x-intercept 0=bx+c --> x = -c/b
        if x(1)<=lb | x(1) >= ub then
            plq = maxoninterval(q1,q2,lb,ub,a,b,c);
        else
            plq_i1 = maxoninterval(q1,q2,lb,x(1),a,b,c);
            plq_i2 = maxoninterval(q1,q2,x(1),ub,a,b,c);
            plq = [plq_i1;plq_i2];
        end
    end;
endfunction


//computes the maximum of two functions under the assumption that the difference of q1 - q2 is quadratic!!!!!
function plq = dq_maxoninterval(q1,q2,lb,ub,a,b,c)
    delta = b^2 - 4*a*c; //calculating the discriminant of the quadratic 
    //printf("\n a: %f, b: %f, c: %f, delta: %f ",a,b,c,delta);
    if delta > 0 then //there are two real roots
    //pause;
        //finding the zeros of qd
        x(1) = (-b + sqrt(delta))/(2*a);
        x(2) = (-b - sqrt(delta))/(2*a);
            x = gsort([x],'r','i');
        if (x(1) < lb & x(2) > ub) | (x(1) > ub & x(2) > ub) | (x(1) < lb & x(2) < lb) then //both zeros are outside our interval
             //plq = maxoninterval(q1,q2,dq,lb,ub);
             plq = maxoninterval(q1,q2,lb,ub,a,b,c);
        elseif x(1) >= lb & x(2) <= ub then //both zeros are inside our interval (NOTE: x(1),x(2) are sorted)
            plq_i1 = maxoninterval(q1,q2,lb,x(1),a,b,c);
            plq_i2 = maxoninterval(q1,q2,x(1),x(2),a,b,c);
            plq_i3 = maxoninterval(q1,q2,x(2),ub,a,b,c);
            plq = [plq_i1;plq_i2;plq_i3];
        elseif x(1) >= lb & x(1) <= ub then //only left root inside interval
          //printf("\n a=%f,b=%f,c=%f, lb=%f, ub=%f, x=%f",a,b,c,lb,ub,x(1));
             plq_i1 = maxoninterval(q1,q2,lb,x(1),a,b,c);
            plq_i2 = maxoninterval(q1,q2,x(1),ub,a,b,c);
            plq = [plq_i1;plq_i2];
        else //only right root inside interval
            plq_i1 = maxoninterval(q1,q2,lb,x(2),a,b,c);
            plq_i2 = maxoninterval(q1,q2,x(2),ub,a,b,c);
            plq = [plq_i1;plq_i2];
        end
    elseif delta < 0 then //there are 2 complex roots (No real roots) 
        plq = maxoninterval(q1,q2,lb,ub,a,b,c);
    else //delta = 0 then there is 1 real root
        x(1) = (-b + sqrt(delta))/(2*a);
        if x(1) <= lb | x(1) >= ub then //the root is outside our lower and upper bound
            plq = maxoninterval(q1,q2,lb,ub,a,b,c);
        else //zero is inside the interval
            plq = maxoninterval(q1,q2,lb,x(1),a,b,c); 
            plq(1,1) = ub; //since the quadratic is tangent to the x-axis, we know that it does not cut the axis
        end
    end    
endfunction



//calculates the max between two functions that do not intersect within the given interval.
function plq = maxoninterval(q1,q2,lb,ub,a,b,c)
    if and(isinf([lb,ub])) then 
        m=0;//pick any point
    else//at most one bound is infinite
        lb1=lb;ub1=ub;
        if lb==-%inf then lb1=ub-1;end;
        if ub==%inf then ub1=lb+1;end;
        m=(lb1+ub1)/2;//midpoint
    end;
  //printf("\n Maxoninterval: q1=[%f,%f,%f], q2=[%f,%f,%f]",q1(1,1),q1(1,2),q1(1,3),q2(1,1),q2(1,2),q2(1,3));
    if a*m^2+b*m+c > 0 then plq = [ub,q1];return; //assumes (q1 - q2) = qld
    else plq = [ub,q2];return;
    end;
endfunction
/////////////FINDS THE MAXIMUM OF TWO PLQ FUCNTIONS///


////PLQ CONVOLUTIONS//////
function [ic_plq] = plq_infconv(plqf,plqg),
  fstar = plq_lft(plqf);
  gstar = plq_lft(plqg);
  fstar_plus_gstar = plq_add(fstar,gstar);
  [ic_plq] = plq_lft(fstar_plus_gstar);
endfunction

function [dc_plq] = plq_deconv(plqf,plqg),
  fstar = plq_lft(plqf);
  gstar = plq_lft(plqg);
  neg_gstar = plq_scalar(gstar,-1);
  fstar_minus_gstar = plq_add(fstar,neg_gstar);
  [dc_plq] = plq_lft(fstar_minus_gstar);
endfunction
////PLQ CONVOLUTIONS//////


///////////////EVALUATES A PLQ FUNCTION ON A GRID///////////////
function [y, k] = plq_eval(plqf,X)
//evaluate a plq function on a grid y=plqf(x), 
//x(k(1,l-1))<=X(k(2,l))<=X(k(3,l))<=x(k(1,l)) i.e. k(:,l)=[i;jl;jr]
    x=plqf(:,1);a=plqf(:,2);b=plqf(:,3);c=plqf(:,4);
    n1=length(x);n=length(X);if size(X,1)<size(X,2) then X=X';end;
    y=[];k=[];
    if (n1 == 1 & x(1) < %inf) then//indicator of a point
        j=find(x==X);
        y=ones(X)*%inf;y(j)=c;
        return;
    end;
    if x(n1) ~= %inf then s=sprintf("Error in plq_eval: right bound of plqf should be infinity instead of %f",X(n));cerror(s);return;end;
    i=1;//index on x
    jl=1;//left index on X
    jr=1;//rigth index on X
    while (jl <= n)
        while (x(i) < X(jl)), i=i+1;end;//x(n)=$inf so always get out
        jr = jl + 1;
         while ((jr <= n) & X(jr) <=x(i)), jr = jr + 1; end;
        if jr > n then jr = n; else jr = jr - 1;end;
        xx = X(jl:jr);
        k(:,$+1)=[i;jl;jr];
        y = [y; a(i)*xx.^2+b(i)*xx+c(i)];
        jl = jr + 1;
        i = i + 1;
    end;
    k(:,1)=[];//remove 1st colum which is [0;0;0]
    if length(y)~=n then s=sprintf("Error in plq_eval: length(X)=%i ~= length(y)=%i",n,length(y));cerror(s);return;end;
endfunction
///////////////EVALUATES A PLQ FUNCTION ON A GRID///////////////


///////////////FIRST ORDER FUNCTION APPROXIMATION///////////////
function plqf = plq_build(x,f,df,needEval,exclude)
//PLQ function = piecewise linear going through (x,f(x)) with derivative df(x) at (x,f(x)) i.e. 1st order approximation of f
//needEval==%f or not entered means the function is vectorized; otherwise evaluate point by point
//NOTE: feval(x,exp) returns an error, but function y=f(x),y=exp(x);endfunction;feval(x,f) works
  if (argn(2) < 3) then df = %f; dy = %f; end;
  if (argn(2) < 4) then needEval = %f; end;  // Assume function is vectorized.
  if (argn(2) < 5) then exclude = %f; end;

  if size(x,1) < size(x,2) then x=x'; end;
  dfType = typeof(df);

  if (needEval) then
    try, y=feval(x,f);
    catch, y=f(x); end;
    if (dfType=="fptr" | df~=%f) then
      try, dy=feval(x,df);
      catch, dy=df(x); end;
    else
      dy = %f;
    end;
  else
    try, y=f(x);
    catch, y=feval(x,f);
    end;
    if (dfType=="fptr" | df~=%f) then
      try, dy=df(x);
      catch, dy=feval(x,df); end;
    else
      dy = %f;
    end;
  end;

  plqf = plq_build_d(x,y,dy,exclude);
endfunction

function plqf = plq_build_d(x,y,dy,exclude)
  //build PLQ function which is piecewise linear using the points (x(i),y(i)) and maybe dy(i)

  if (argn(2) < 3) then dy = %f; end;
  if (argn(2) < 4) then exclude = %f; end;

  infl = %f;
  infr = %f;
  dyType = typeof(dy);
  if (dyType~="fptr" & dy==%f) then
    if (exclude) then
      while (y(1) == %inf), infl = %t; x = x(2:$);   y = y(2:$);   end;
      while (y($) == %inf), infr = %t; x = x(1:$-1); y = y(1:$-1); end;
    end;
    dy = (y(2:$) - y(1:$-1)) ./ (x(2:$) - x(1:$-1));
    dym=[dy(1);dy(1:$)];//left derivative at (xm,ym)
    plqf=[x,zeros(x),dym,y-dym.*x];
    plqf=[plqf;%inf,0,dy($),y($)-dy($)*x($)];//careful PLQ is represented with a x^2+b x + c
    if (exclude) then
      if (infl) then plqf(1,2:4) = [0,0,%inf]; end;
      if (infr) then plqf($,:) = [%inf,0,0,%inf]; end;
    end;
  else
    //disp([x,y,dy], "initial [x,y,dy]:");
    while (y(1) == %inf | dy(1) == %inf), infl = x(1); x = x(2:$);   y = y(2:$);   dy = dy(2:$);   end;
    while (y($) == %inf | dy($) == %inf), infr = x($); x = x(1:$-1); y = y(1:$-1); dy = dy(1:$-1); end;
    //disp([x,y,dy], "infs removed [x,y,dy]:");
    //when dy(i)==dy(i+1) remove the point x(i) since it does not give any 1st order info
    ni = find(dy(2:$)-dy(1:$-1)~=0);
    ni=[ni,length(x)];//always keep the rightmost point
    x=x(ni);y=y(ni);dy=dy(ni);
    //disp([x,y,dy], "dups removed [x,y,dy]:");
    //(xm,ym) = intersection of tangent at (x(i),y(i)) and at (x(i+1),y(i+1))
    xm=(y(1:$-1)-y(2:$)+dy(2:$).*x(2:$)-dy(1:$-1).*x(1:$-1))./(dy(2:$)-dy(1:$-1));
    ym=y(1:$-1)+dy(1:$-1).*(xm-x(1:$-1));
    dym=dy(1:$-1);//left derivative at (xm,ym)
    //disp([xm,ym,dym], "intersection of tangents [xm,ym,dym]:");
    plqf=[xm,zeros(xm),dym,ym-dym.*xm;%inf,0,dy($),y($)-dy($)*x($)];//careful PLQ is represented with a x^2+b x + c
    //handle %inf values or derivatives
    if (infl ~= %f) then
      if (exclude) then
        plqf = [x(1),0,0,%inf; plqf];
      else
        plqf = [infl,0,0,%inf; plqf];
      end;
    end;
    if (infr ~= %f) then
      if (exclude) then
        plqf($,1) = x($);
      else
        plqf($,1) = infr;
      end;
      plqf = [plqf; %inf,0,0,%inf];
    end;
  end;
  plqf=plq_clean(plqf);//clean up, remove linear parts
endfunction


///////////////FIRST ORDER FUNCTION APPROXIMATION///////////////

///////////////PIECEWISE LINEAR QUADRATIC MOREAU ENVELOPE (WRAPPER) ///////////////
function plqme = plq_me(plqf,lambda,isConvex)
  if argn(2)<=2 then isConvex=%t;end;//default to convex functions

  a = plqf(:,2);
  if (lambda < 0 | a(1) < 0 | a($) < 0) then
    maxScale = plq_me_max_scale(plqf);
    if (lambda < 0 | lambda > maxScale) then
      cerror(sprintf("plq_me(): Scale %f should be in (0,%f].", lambda, maxScale));
      plq_me = [];
      return;
    end;
  end;

//    x=plqf(:,1);a=plqf(:,2);b=plqf(:,3);c=plqf(:,4);
//    //if size(x,1) == 1 & a(1) == 0 & b(1) == 0 then //we have an indicator function of a point [x, 0, 0, c] where x is the horizontal pos, and c is the height.
//    if size(x,1) == 1 & x(1) ~= %inf then //we have an indicator function of a point [x, 0, 0, c] where x is the horizontal pos, and c is the height.
//      printf("plq_me of indicator:");
//      disp(plqf);
//      plqf = [x, 0, 0, a*x^2 + b*x + c];
//      plqf = [x, lambda*a, lambda*b, lambda*c + (1/2)*x^2];
//    else
//      printf("plq_me of function:");
//      disp(plqf);
//      plqf = [x, lambda*a + (1/2), lambda*b, lambda*c];
//    end

    plqme = plq_scalar(plqf,lambda);
    plqme = plq_add(plqme,[%inf,1/2,0,0]);
    plqme = plq_lft(plqme,isConvex);
    sx=plqme(:,1);sa=plqme(:,2);sb=plqme(:,3);sc=plqme(:,4);
//    plqme = [sx, (-1/lambda)*sa + (1/(2*lambda)), (-1/lambda)*sb, (-1/lambda)*sc];
    plqme = plq_scalar(plqme,-1);
    plqme = plq_add(plqme,[%inf,1/2,0,0]);
    plqme = plq_scalar(plqme,1/lambda);
endfunction

function scale = plq_me_max_scale(varargin)
  n = length(varargin);
  scale = %inf;
  for i = 1:n
    f = varargin(i);
    a = f(:,2);
    b = f(:,3);
    if (a(1) < 0) then scale = min(scale, -1/2/a(1)); end;
    if (a($) < 0) then scale = min(scale, -1/2/a($)); end;
  end;
endfunction
///////////////PIECEWISE LINEAR QUADRATIC MOREAU ENVELOPE (WRAPPER) ///////////////


///////////////PLQ SCALAR MULTIPLCATION FUNCTION///////////////////////////////////////////////////////////////
function plqf = plq_scalar(plqfs,lambda)
  x=plqfs(:,1);a=plqfs(:,2);b=plqfs(:,3);c=plqfs(:,4);
  plqf = [x, lambda*a, lambda*b, lambda*c];
endfunction
///////////////PLQ SCALAR MULTIPLCATION///////////////////////////////////////////////////////////////


////////////////////////////////////////PROXIMAL AVERAGE///////////////////////////////////////////////////////
function pa = plq_pa(f1,f2,lambda_pa,isConvex)
  if argn(2)<=3 then isConvex=%t;end;//default to convex functions
//f[lambda] = (lambda(f1 + (1/2)normsquared)* + (1 - lambda)(f2 + (1/2)normsquared)*)* - (1/2)normsquared
    normsquared = [%inf, 1/2, 0, 0];
    negnormsquared = plq_scalar(normsquared,-1);
    f1normsquared = plq_add(f1,normsquared);
    f2normsquared = plq_add(f2,normsquared);
    f1normsquaredstar = plq_lft(f1normsquared,isConvex);
    f2normsquaredstar = plq_lft(f2normsquared,isConvex);
    f1normsquaredstarlambda = plq_scalar(f1normsquaredstar,1-lambda_pa);//1-lambda first function
    f2normsquaredstarlambda = plq_scalar(f2normsquaredstar,lambda_pa);
    proxadd = plq_add(f1normsquaredstarlambda,f2normsquaredstarlambda);
    proxaddstar = plq_lft(proxadd);
    pa = plq_add(proxaddstar,negnormsquared);
endfunction

function pa = plq_pa_mu(mu, lambdas, varargin)
  // 0 <= mu <= 1
  // plq_pa_mu(mu, 0..1, f1, f2)
  // plq_pa_mu(mu, [0.3,0.7], f1, f2)
  // plq_pa_mu(mu, [0.1,0.3,0.6], f1, f2, f3)
  //printf("pla_ma_mu(%.2f, %.2f)\n", mu, lambdas(1));

  n = length(varargin);

  if (length(lambdas) == 1 & length(varargin) == 2) then
    lambdas = [1-lambdas, lambdas];
  end;
  if (length(lambdas) ~= n) then
    cerror("plq_pa_mu: length(lambdas) must equal the number of functions.  (Or have one lambda, two functions.)");
    pa = []; return;
  end;
  if (abs(sum(lambdas) - 1) > 1E-6) then
    cerror("plq_pa_mu: sum(lambdas) must equal 1.");
    pa = []; return;
  end;

  pa = plq_scalar(plq_me(varargin(1), mu, %f), lambdas(1));
  for i = 2:n
    f = varargin(i);
    if (lambdas(i) ~= 0) then
      pa = plq_add(pa, plq_scalar(plq_me(f, mu, %f), lambdas(i)));
    end;
  end;
  pa = plq_scalar(plq_me(plq_scalar(pa, -1), mu, %f), -1);
endfunction
////////////////////////////////////////PROXIMAL AVERAGE///////////////////////////////////////////////////////


///////////////////////////////////////PLQ CONJUGATE///////////////////////////////////////
function plqfstar = mikeplq_lft(plqf) 
    x=plqf(:,1);a=plqf(:,2);b=plqf(:,3);c=plqf(:,4);
    n = size(x,1); //number of rows
    i=1;j=1; //index
    while i <= n, 
        plqType = getType(n,x,a,b,c,i);
        //printf("\n%d --- TYPE ---- \n",plqType);
        select plqType,
        case 6 then//BOUNDED RHS
          s(i) = %inf;
          sa(i) = 0;
          sb(i) = x(i-1);
          sc(i) = (-1)*a(i-1)*x(i-1)^2-b(i-1)*x(i-1)-c(i-1);
          j = j + 1;
        case 5 then//BOUNDED LHS
          s(i) = 2*a(i+1)*x(i) + b(i+1);
          sa(i) = 0;
          sb(i) = x(i);
          sc(i) = (-1)*a(i+1)*x(i)^2-b(i+1)*x(i)-c(i+1);
          j = j + 1;
        case 3 then//LINE - TO - POINT
            s(i) = b(i);sa(i) = 0;sb(i) = 0;sc(i) = -c(i);
        case 4 then//POINT - TO - LINE
            s(i) = %inf;sa(i) = 0;sb(i) = x(i);sc(i) = -c(i);
        case 1 then//linear - TO - quadratic or linear
        //first row of conjugate matrix
        if i < n then 
                s(j) = b(i);
        else
                s(j) = %inf;
        end
            sa(j) = 0;sb(j) = 0;sc(j) = %inf;
            
        //second row of conjugate matrix
        if i < n then
            if s(j) < 2*a(i+1)*x(i) + b(i+1) then
                s(j+1) = 2*a(i+1)*x(i) + b(i+1);
                sa(j+1) = 0;
                sb(j+1) = x(i);
                sc(j+1) = (-1)*b(i)*x(i) - c(i);
                j = j + 2;
            else j = j + 1;
            end
        end
        case 2 then//quadratic - TO - quadratic or linear
            //first row of conjugate matrix
            if i < n then
                s(j) = 2*a(i)*x(i) + b(i);
            else 
                s(j) = %inf;
            end
            sa(j) = 1/(4*a(i));
            sb(j) = (-1/2)*(b(i)/a(i));
            sc(j) = (1/4)*(b(i)*b(i))/a(i) - c(i); //potential for infinity value

            //second row of conjugate matrix
            if (i < n) then
                if s(j) < 2*a(i+1)*x(i) + b(i+1) then
                    s(j+1) = 2*a(i+1)*x(i) + b(i+1);
                    sa(j+1) = 0;
                    sb(j+1) = x(i);
                    sc(j+1) = -a(i)*x(i)*x(i) - b(i)*x(i) - c(i);
                    j = j + 2;
                else j = j + 1;
                end
            end
        end
        i = i + 1;
    end
    plqfstarDirty = [ s , sa , sb , sc ];
    plqfstar = plq_clean(plqfstarDirty);
endfunction
///////////////////////////////////////PLQ CONJUGATE///////////////////////////////////////

///////////////////////////////////////PLQ CONJUGATE///////////////////////////////////////
function plqfstar = plq_lft(plqf,isConvex)
  if argn(2)<=1 then isConvex=%t;end;//default to convex functions

  if ~isConvex then plqf=plq_co(plqf);end;
    x=plqf(:,1);a=plqf(:,2);b=plqf(:,3);c=plqf(:,4);
    n = size(x,1); //number of rows
    i=1;//primal index: x(i); scan plqf
    j=1; //dual index: s(j); build plqfstar
    //special case: Indicator of a point or linear function
    if n == 1 then
        if  x(i) == %inf & a == 0 then//LINE - TO - POINT
            plqfstar=[b,0,0,-c];return;
        elseif x(i) < %inf then //POINT - TO - LINE
               plqfstar=[%inf,0,x,-c];return;
        end
    end
    
    if c(1) == %inf then //domain is left-bounded: add a linear part
        [s(1),sa(1),sb(1),sc(1)]=(2*a(2)*x(1) + b(2),0,x(1),-a(2)*x(1)^2-b(2)*x(1)-c(2));
        j = j + 1;i = i + 1;
    elseif a(1)==0 then //first part is linear: add infinity + linear part
        [s(1),sa(1),sb(1),sc(1)]=(b(1),0,0,%inf);
        [s(2),sa(2),sb(2),sc(2)]=(2*a(2)*x(1) + b(2),0,x(1),-b(1)*x(1) - c(1));
        //since -b(1)*x(1) - c(1)=-a(2)*x(1)^2-b(2)*x(1)-c(2) the formula is the same as for left-bounded above
        j = j + 2;i = i + 1;
    end
    //Now each line in plqf(i:n-1,:) gives a quadratic part + a linear part

    //index of the last row considering there are 3 starting cases
    //i=j=1; i=j=2; and i=2;j=3; 
    //and each value of i between istart and n-1 gives 2 values for j
    jend=2*(n-i)+j-1;//n-i=(n-1)-(i-1)=number of primal values to read; j-1=dual values already added

    if jend > j then//when there is a middle part
        jq=j:2:jend-1;//index for quadratic parts
        jl=j+1:2:jend;//index for linear parts
        ieee(2);//allow division by 0
        ai=a(i:n-1);xi=x(i:n-1);bi=b(i:n-1);ci=c(i:n-1);t=4*ai;//tmp vars to reduce calculations
        sq=2*ai.*xi+bi;saq=t.^(-1);sbq=-2*bi./t;scq=(bi.^2)./t-ci;//quadratic part (careful 1./t<>t.^(-1))    
        sl=2*a(i+1:n).*xi+b(i+1:n);sal=zeros(xi);sbl=xi;scl=-ai.*xi.^2-bi.*xi-ci;//linear part
        ieee(0);//disallow division by 0    
        z=zeros(jend-j+1,1);s(j:jend)=z;sa(j:jend)=z;sb(j:jend)=z;sc(j:jend)=z;
        s(jq)=sq;sa(jq)=saq;sb(jq)=sbq;sc(jq)=scq;
        s(jl)=sl;sa(jl)=sal;sb(jl)=sbl;sc(jl)=scl;
        j=jend+1;
    end;
    if c(n) == %inf then//domain is right-bounded: add a linear part
        s(j) = %inf;
        sa(j) = 0;
        sb(j) = x(n-1);
        sc(j) = -a(n-1)*x(n-1)^2-b(n-1)*x(n-1)-c(n-1);
    elseif a(n)==0 then//last part is linear: add infinity
        s(j) = %inf;
        sa(j) = 0;
        sb(j) = 0;
        sc(j) = %inf;
    else //last part is quadratic: add a quadratic
        s(j) = 2*a(n)*x(n) + b(n);    
        sa(j) = 1/(4*a(n));
        sb(j) = -b(n)/(2*a(n));
        sc(j) = b(n)^2/(4*a(n)) - c(n);//always finite value since a(n)>0
    end
    j = j + 1;i = i + 1;
    plqfstar = plq_clean([ s , sa , sb , sc ]);
endfunction
///////////////////////////////////////PLQ CONJUGATE///////////////////////////////////////


//////////PLQ CONJUGATE HELPER FUNCTION//////////////
function plqType=getType(n,x,a,b,c,i)
    if a(i) == 0 then
        plqType = 1;//part is linear
    else plqType = 2;//part is quadratic
    end
    
    if n == 1 & x(i) == %inf & a(i) == 0 then
        plqType = 3;//LINE - TO - POINT
    elseif n == 1 & x(i) < %inf then //POINT - TO - LINE
       plqType = 4;
    end
    
    if a(i) == %inf | b(i) == %inf | c(i) == %inf & i == 1 then 
      plqType  = 5; //Bounded by the LHS
    end
    
    if a(i) == %inf | b(i) == %inf | c(i) == %inf & i == size(x,1) then 
      plqType  = 6; //Bounded by the RHS
    end
endfunction
//////////PLQ CONJUGATE HELPER FUNCTION//////////////


//////////PLQ CONJUGATE HELPER FUNCTION//////////////
function [plqClean] = plq_clean(plqDirty)  
  eps=1E-8;//precision below which numbers are rounded to zero
  dplq=plqDirty(2:$-1,:)-plqDirty(1:$-2,:);
  //clear duplicate point values e.g [0,1,0,1;0,2,0,0] -> [0,1,0,1]
  j1=find(abs(dplq(:,1))<eps);//delete j1+1
  if length(j1)>0 then plqDirty(j1+1,:)=[];end;
  
  dplq=plqDirty(2:$,:)-plqDirty(1:$-1,:);
  //clear duplicate rows e.g. [0,1,0,0;0,1,0,0]->[0,1,0,0]
  if size(dplq,1)==1 then
    if norm(dplq(1,2:$))<eps then plqDirty(1,:)=[]; end;
  else
    m=abs(dplq(:,2:$))*ones(3,1);j2=find(m<eps);
    plqDirty(j2,:)=[];    
  end
  //round small numbers to zero
  //this should be done by the clean function
  //but clean is buggy when the matrix contains %inf 
  //P1=plqDirty(:,2:$)
  P1=plqDirty
  P=matrix(P1,length(P1))
  ni=find(abs(P)<eps)
  P(ni)=0
  P1=matrix(P,size(P1))
  //plqDirty(:,2:$)=P1;
  plqDirty=P1;
  //Mike
  //Addition of infinity values results in functions like
  //  [x x x x]
  //  [x x x %inf]
  //  [x x x x%inf]
  //  [%inf x  x %inf]
  //However it is properly represented as
  //  [x x x x]
  //  [%inf x  x %inf]
  //The following lines of code remove the redundant infinity rows.

  plqInf = _plq_rowalwaysinf(plqDirty);
  [rs,cs] = find(and([plqInf(1:$-1,:), plqInf(2:$,:)], 'c'))
  if (size(rs,1) ~= 0) then
    plqDirty(rs+1,2:$-1) = 0;  // If rs==[], then rs+1==[1],
    plqDirty(rs,:) = [];       //  we want rs+1==[], hence the if.
  end

  plqClean = plqDirty;
endfunction

// A polynomial is always infinite if a and b are not infinite
// and not NaN, and c is infinite.
function [alwaysinf, sgn] = _plq_rowalwaysinf(plqf)
  alwaysinf = ~(or(isinf(plqf(:,2:$-1)), 2) ...
                | or(isnan(plqf(:,2:$-1)), 2)) ...
              & isinf(plqf(:,$));
  sgn = sign(plqf(:,$));
endfunction

function plq_plotpa(plq0,plq1,x,lset,rect,isConvex)
//plot the graph of (x,lambda)->pa(f,lambda)
//2d graph with colors for each value of lambda
//plq0, plq1: plq functions
//x: column vector
//lset: row vector; lset is the lambda set i.e. the values of lambda from 0 to 1
//plot in the current graphic window

//EXAMPLE
// x=(-2:0.2:2)';
// function y=f0(x),y=(x+1).^2+2;endfunction
// function dy=df0(x),dy=2*(x+1);endfunction
// 
// function y=f1(x),y=(x-1).^2;endfunction
// function dy=df1(x),dy=2*(x-1);endfunction
// plq0=plq_build(x,f0,df0);
// plq1=plq_build(x,f1,df1);
// 
// lset=(0:0.1:1);rect=[-2,-1,2,4];
// //plq0=[%inf,0.5,0,0];plq1=[0,0,0,1];//quadratic to indicator
// 
// 
// plq_plotpa(plq0,plq1,x,lset,rect);

  if (argn(2) < 6) then isConvex = %t; end;
  if size(x,2) >1 then x=x';end;
  if size(lset,1) >1 then lset=lset';end;
  clf();th=1;driver("Rec");//we want to zoom so use Rec driver not X11
  xset("pixmap",1);xset("wwpc");//plot in memory for faster display

  nc=max(round(length(lset)/100),100);//number of colors
  cm=[jetcolormap(nc)];xset("colormap",cm);
  colorbar(0,1);//h=gce();h.parent.font_size=4;

  i=1;stp=ceil(length(lset)/10);
  beginProgress();
  for lambda=lset,
      pa=plq_pa(plq0,plq1,lambda,isConvex);
      y=plq_eval(pa,x);
      plot2d(x,y,style=ceil(lambda*nc)+1,rect=rect);//lambda no longer equispaced so cannot use style=i
      //if (i==1) then h=gca(); h.font_size=4; end;
      if modulo(i,stp)==0 then
        xset("wshow");
        showProgress(i/length(lset));
      end;
      i=i+1;
  end;
  finishProgress();
  xset("wshow");
  xset("pixmap",0);
endfunction

function plq_plotpa_lambdaSet(plq0,plq1,x,lset,mu,rect)
  // Plot the graph of (x,lambda)->pa(f,lambda), but use plq_pa_mu which
  // supports the proximal average of non-convex functions.  Varies lambda
  // with a fixed mu.

  if size(x,2) >1 then x=x';end;
  if size(lset,1) >1 then lset=lset';end;
  clf();th=1;driver("Rec");//we want to zoom so use Rec driver not X11
  xset("pixmap",1);xset("wwpc");//plot in memory for faster display

  nc=max(round(length(lset)/100),100);//number of colors
  cm=[jetcolormap(nc)];xset("colormap",cm);
  colorbar(0,1);//h=gce();h.parent.font_size=4;

  i=1;stp=ceil(length(lset)/10);
  beginProgress();
  for lambda=lset,
    pa=plq_pa_mu(mu,lambda,plq0,plq1);
    y=plq_eval(pa,x);
    plot2d(x,y,style=ceil(lambda*nc)+1,rect=rect);//lambda no longer equispaced so cannot use style=i
    //if (i==1) then h=gca(); h.font_size=4; end;
    if modulo(i,stp)==0 then
      xset("wshow");
      showProgress(i/length(lset));
    end;
    i=i+1;
  end;
  finishProgress();
  xset("wshow");
  xset("pixmap",0);
endfunction

function plq_plotpa_muSet(plq0,plq1,x,lambda,mus,rect)
  // Plot the graph of (x,lambda)->pa(f,lambda), but use plq_pa_mu which
  // supports the proximal average of non-convex functions.  Varies mu
  // with a fixed lambda.

  if size(x,2) >1 then x=x';end;
  if size(mus,1) >1 then mus=mus';end;
  clf();th=1;driver("Rec");//we want to zoom so use Rec driver not X11
  xset("pixmap",1);xset("wwpc");//plot in memory for faster display

  nc=max(round(length(mus)/100),100);//number of colors
  cm=[jetcolormap(nc)];xset("colormap",cm);
  colorbar(0,1);//h=gce();h.parent.font_size=4;

  i=1;stp=ceil(length(mus)/10);
  beginProgress();
  for mu=mus,
    pa=plq_pa_mu(mu,lambda,plq0,plq1);
    y=plq_eval(pa,x);
    plot2d(x,y,style=ceil(lambda*nc)+1,rect=rect);//lambda no longer equispaced so cannot use style=i
    //if (i==1) then h=gca(); h.font_size=4; end;
    if modulo(i,stp)==0 then
      xset("wshow");
      showProgress(i/length(mus));
    end;
    i=i+1;
  end;
  finishProgress();
  xset("wshow");
  xset("pixmap",0);
endfunction

//////////PLQ CONJUGATE HELPER FUNCTION//////////////


///////////PLQ CONVEX HULL/////////////////////////
//TODO, (1) UPDATE PLQ_EVAL FOR NONCONVEX FUNCTIONS?.
//      (2) UPDATE PLQ_BUILD FOR NONCONVEX FUNCTIONS.
////////////////////////////////////////////////////////////////////////////
//PROBLEM 1. (1) f1'(x2) = f2'(x4), 
//           (2) m1(x-x2) + f1(x2) = m2(x-x4) + f2(x4), where
//           (3) x1 < x2 < x3
//           (4) x3 < x4 < x5
//
//PROBLEM 2. (1) m1(x-x2) + f1(x2) = m1(x-x5) + f2(x5), where
//           (2) x1 < x2 < x3
//           (3) x3 < x4 < x5, and
//           (4) m1=f1'(x2)
//
//PROBLEM 3. (1) m2(x-x1) + f1(x1) = m2(x-x4) + f2(x4), where
//           (2) x1 < x2 < x3
//           (3) x3 < x4 < x5, and
//           (4) m2=f2'(x4)
//
//PROBLEM 4. (1) m1(x-x1) + f1(x1) = m1(x-x5) + f2(x5), where
//           (2) x1 < x2 < x3
//           (3) x3 < x4 < x5
//
////////////////////////////////////////////////////////////////////////////

//Quick plot of one or two PLQ functions.
//x1,x5 can be finite or infinity values. 
function plq_plot(plqf1,plqf2,x1,x5),
    if (argn(2) < 2) then plqf2 = %f; end;
    if (argn(2) < 3 | x1 == -%inf) then x1 = -5; end;
    if (argn(2) < 4 | x5 == %inf) then x5 = 5; end;
    x=linspace(x1,x5,200)';
    clf();
    plqf1_e = [x1,0,0,%inf;plqf1;%inf,0,0,%inf];
    y1=plq_eval(plqf1_e,x);
    if (~plqf2) then
      plot2d(x,y1);
    else
      plqf2_e = [x1,0,0,%inf;plqf2;%inf,0,0,%inf];
      y2=plq_eval(plqf2_e,x);
      plot2d(x,[y1,y2]);
    end;
    a=gca(); // Handle on axes entity
    a.thickness=3;
    if (plqf2) then
      poly1= a.children.children(1); //store polyline handle into poly1
      poly1.foreground = 6; // another way to change the style
      poly1.thickness = 4;
      poly1.line_style = 2;
    end;
endfunction;

//Plot multiple PLQ functions.
//x1,x5 can be finite or infinity values. 
function plq_plot2(x1,x5,varargin)
    if (argn(2) < 1 | x1 == %f | x1 == -%inf) then x1 = -5; end;
    if (argn(2) < 2 | x5 == %f | x5 == %inf) then x5 = 5; end;
    if (length(varargin) == 0) then return; end;
    x=linspace(x1,x5,200)';
    y=[];
    // Todo: Plot indicator functions specially: plot2d(f(1,1), f(1,4), -5);
    for i = 1:length(varargin)
      f = varargin(i);
      y(:,i) = plq_eval(f, x);
    end;
    plot2d(x, y);
endfunction;

function plqco=plq_co(plqf)
    EPS=1E-10;
    plqf_original=plqf; plqfLHS=[];plqfRHS=[];
    
    //Bounded plq functions will have an infinity value as a coefficient in either
    //the first row or the last row, simply remove these pieces and add these pieces
    //after the convex hull has been computed.
    if or(plqf(1,2:$) == %inf) then plqfLHS = plqf(1,:); plqf=plqf(2:$,:); end;
    if or(plqf($,2:$) == %inf) then plqfRHS = plqf($,:); plqf=plqf(1:$-1,:); end;
    
    n = size(plqf,1);
    x=plqf(:,1);a=plqf(:,2);b=plqf(:,3);c=plqf(:,4); //update the coefficients
    if a(1) < 0  | a(n) < 0 | (a(1)==0 & a(n) == 0 & b(n) < b(1) ) then //special case
      plqco=[%inf,0,0,-%inf];
      return;
    end;
    
   
    i=1;
    while i <= n-1 & n >= 2, 
        a=clean(a(:,1)); b=clean(b(:,1)); c=clean(c(:,1));
        //Considering a PLQ function with two pieces, it is nonconvex if the following condition holds.
        if (2*a(i)*x(i)+b(i)) - (2*a(i+1)*x(i)+b(i+1)) > EPS |  a(i) < 0 | a(i+1) < 0 then
            if n >=3 & i >= 2 then x1 = plqf(i-1,1); else x1 = -%inf; end; 
            f1=plqf(i,:); f2=plqf(i+1,:);x3=f1(1,1); x5=f2(1,1);
            //printf("\n f1=[%f,%f,%f,%f],f2=[%f,%f,%f,%f]; x1=%f;x3=%f;x5=%f",f1(1,1),f1(1,2),f1(1,3),f1(1,4),f2(1,1),f2(1,2),f2(1,3),f2(1,4),x1,x3,x5);
            //plq_plot(plqf_original,plqf,-%inf,%inf); 
            plqf = [plqf(1:(i-1),:);plq_conv_on_interval(f1,f2,x1,x3,x5);plqf((i+2):$,:)];
            plqf = plq_clean(plqf);
            //plq_plot(plqf_original,plqf,-%inf,%inf);
            if n <= 1 then plqco=[plqf]; break; end; 
            if i > 1 then i=i-1; end; //may need to jump back if there is a nonconvex part. (not sure how to get around this)
    else
        i=i+1;
    end;
    n = size(plqf,1); //update size of the PLQ function
    x=plqf(:,1);a=plqf(:,2);b=plqf(:,3);c=plqf(:,4); //update the coefficients
    end;
    plqco=[plqfLHS;plqf;plqfRHS];
endfunction  

//We assume x(1) < x(2) < x(3) < x(4) < x(5) are ordered and finite (no infinity values). 
//In creating the PLQ hull we may need to create points x(2) and x(4).
//Note that x(1) is the LEFTBOUND, x(3) is the partition point, and x(5) is the RIGHTBOUND in our PLQ interval. 
//plq_conv_on_interval finds the convex hull on the interval [x1,x5].
function [plqco] = plq_conv_on_interval(f1,f2,x1,x3,x5) //Assume f1 is to the left of f2
  x(1) = x1; a(1)=f1(1,2); a(2)=f2(1,2);
  x(3) = x3; b(1)=f1(1,3); b(2)=f2(1,3);
  x(5) = x5; c(1)=f1(1,4); c(2)=f2(1,4);
  ieee(2); //allow infinity i.e. 1/0
  plqco=[];
  
  if a(1) < 0 then  //concave down pieces are simply replaced by a linear function.
      f1=_plq_conv_buildl(x,a,b,c,1,1,1,3);
      f2=[x(5),a(2),b(2),c(2)];
      plqco=plq_conv_on_interval(f1,f2,x(1),x(3),x(5));
  elseif a(2) < 0 then
      f1=[x(3),a(1),b(1),c(1)];
      f2=_plq_conv_buildl(x,a,b,c,2,5,2,3);
      plqco=plq_conv_on_interval(f1,f2,x(1),x(3),x(5));
  elseif 2*a(1)*x(3)+b(1) <= 2*a(2)*x(3)+b(2) then //trivial case
      plqco=[x(3),a(1),b(1),c(1);x(5),a(2),b(2),c(2)];
  elseif a(1) == 0 & a(2) == 0 then //(LINEAR-LINEAR) f1 is linear and f2 is linear.
      [plqco]=_plq_conv_buildl(x,a,b,c,1,1,2,5);
  elseif a(1) == 0 & a(2) <> 0 then // (LINEAR-QUDARTIC) if f1 is linear and f2 is quadratic then x(4) is determined.
      plqco=[_conv_interval_func(x,a,b,c,x1,x3,x5)];
  elseif a(1) <> 0 & a(2) == 0 then // (QUADRATIC-LINEAR) if f1 is quadratic and f2 is linear then x(2) is determined.
      plqco=[_conv_interval_func(x,a,b,c,x1,x3,x5)];
  elseif a(1) <> 0 & a(2) <> 0 then // (QUADRATIC-QUADRATIC) if f1 is quadratic and f2 is quadratic we need to find solutions to A*x(4)^2 + B*x(4) + C.
      //The following code solves [PROBLEM 1.], 
      A=(1/4)*(-4*a(2)^2+4*a(2)*a(1))/a(1);
      B=(1/4)*(-4*a(2)*b(2)+4*b(1)*a(2))/a(1);
      C=(1/4)*(-b(2)^2-b(1)^2+4*c(1)*a(1)+2*b(1)*b(2)-4*c(2)*a(1))/a(1);
      D = B^2 - 4*A*C; //Discriminant.
      //printf("QUADRATIC-QUADRATIC CASE: Solving %f x^2 + %f x + %f, with discriminant=%f\n",A,B,C,D);
      if A == 0 then //Linear case, so we have Bx+C=0 with one zero.
          x(4) = (-C/B);
          x(2) = (-1/2)*(b(1)-2*a(2)*x(4)-b(2))/a(1);
          if x(2) < x(1) | x(3) < x(2) then x(2)=%inf; end;
          if x(4) < x(3) | x(4) > x(5) then x(4)=%inf; end;
      //if x(2) or x(4) are outside the interval, send the x(i) to infinity. In otherwords these solutions provide no information
      //and thus we should just ignore them. Similarly if D < 0, just ignore x(2) and x(4) as they don't help with the hull. 
      else // A <> 0, Quadratic case, so we have Ax^2+Bx+C=0, with one or two zeros. Solving for x(4).
            if D>0 then
                nr = (-B - sqrt(D))/(2*A); //negative root
                pr = (-B + sqrt(D))/(2*A); //positive root
                if D==0 then nr=pr; end;
                if x(3) < pr & pr < x(5) then x(4) = pr;
                elseif x(3) < nr & nr < x(5) then x(4) = nr; //printf(" NEGATIVE ROOT ");
                else x(4)=%inf; end;
                x(2) = (-1/2)*(b(1)-2*a(2)*x(4)-b(2))/a(1);
                if x(1) > x(2) | x(2) > x(3) then x(2)=%inf; end;
            else
                x(2)=%inf; x(4)=%inf;
            end
      end
      if x(2) < %inf & x(4) < %inf then
          plqco=[x(2),a(1),b(1),c(1);_plq_conv_buildl(x,a,b,c,1,2,2,4);x(5),a(2),b(2),c(2)];
      else //x(2) == %inf | x(4) == %inf, 
         //Here we solve the SECOND PROBLEM: as x(2) or x(4) is infeasible in [PROBLEM 1.]
          plqco=[_conv_interval_func(x,a,b,c,x1,x3,x5)];
      end;
  else cerror("Case does not exist.");
  end
endfunction



//if we can't find a solution to [PROBLEM 1.] then we need to find a solution to [PROBLEM 2.].
//The following routine solves [PROBLEM 2.].
function [plqco] = _conv_interval_func(x,a,b,c,x1,x3,x5),
        EPS=1E-10; x(1)=x1;x(3)=x3;x(5)=x5;
    
        //solving the quadratic for x(2)
        //if a(1) <> 0 & x(5) < %inf & x(1) > -%inf then
        if a(1) <> 0 & x(5) < %inf then
            A1 = -a(1);
            B1 = 2*a(1)*x5;
            C1 = c(1) + b(1)*x5 - a(2)*x5^2 - b(2)*x5 - c(2);
            D1 = B1^2 - 4*A1*C1;
            pr1 = (-B1 + sqrt(D1))/(2*A1); //solution is x2  
            nr1 = (-B1 - sqrt(D1))/(2*A1); //negative root required for the LINEAR-QUADRATIC CASE.
            
            if x(1) <= pr1 & pr1 <= x(3) then x(2) = pr1; plqco=[x(2),a(1),b(1),c(1);_plq_conv_buildl(x,a,b,c,1,2,2,5)];
            elseif x(3) <= pr1 & pr1 <= x(5) then x(4)=pr1; plqco=[_plq_conv_buildl(x,a,b,c,1,1,2,4);x(5),a(2),b(2),c(2)]; //potentially useless
            elseif x(1) <= nr1 & nr1 <= x(3) then x(2) =pr1; plqco=[x(2),a(1),b(1),c(1);_plq_conv_buildl(x,a,b,c,1,2,2,5)];
            elseif x(3) <= nr1 & nr1 <= x(5) then x(4)=pr1; plqco=[_plq_conv_buildl(x,a,b,c,1,1,2,4);x(5),a(2),b(2),c(2)]; //potentially useless
            else plqco=_plq_conv_buildl(x,a,b,c,1,1,2,5);
            end; 
        //solving the quadratic for x(4)
        //elseif a(2) <> 0 & x(1) > -%inf & x(5) < %inf then
        elseif a(2) <> 0 & x(1) > -%inf then
            A2 = a(2);
            B2 = -2*a(2)*x(1);
            C2 = -b(2)*x(1) + a(1)*x(1)^2 + b(1)*x(1) + c(1) - c(2);
            D2 = B2^2 - 4*A2*C2;
            pr2 = (-B2 + sqrt(D2))/(2*A2);  
            nr2 = (-B2 - sqrt(D2))/(2*A2);
            
            if x(3) <= pr2 & pr2 <= x(5)  then x(4)=pr2; plqco=[_plq_conv_buildl(x,a,b,c,1,1,2,4);x(5),a(2),b(2),c(2)];
            elseif x(1) <= pr2 & pr2 <= x(3)  then x(2)=pr2; plqco=[x(2),a(1),b(1),c(1);_plq_conv_buildl(x,a,b,c,1,2,2,5)]; //potentially useless
            elseif x(3) <= nr2 & nr2 <= x(5)  then x(4)=nr2; plqco=[_plq_conv_buildl(x,a,b,c,1,1,2,4);x(5),a(2),b(2),c(2)];
            elseif x(1) <= nr2 & nr2 <= x(3)  then x(2)=nr2; plqco=[x(2),a(1),b(1),c(1);_plq_conv_buildl(x,a,b,c,1,2,2,5)]; //potentially useless
            else plqco=_plq_conv_buildl(x,a,b,c,1,1,2,5);
            end;
        else plqco=_plq_conv_buildl(x,a,b,c,1,1,2,5); 
        end;
        
        

endfunction

//build a linear function, f1i, is the function index, and x1i is the x-value we wanted to evaluate the function on.
//similarly f2i is the function index of the second function, and x2i corresponds to the value we evaluate the function on.
//In other words constructs a linear function from (x1i,f[f1i](x(x1i))), to (x(x2i),f[f2i](x(x2i))).
//function [plq_fctn] = _plq_conv_build2(x,a,b,c)
function [l_fctn]=_plq_conv_buildl(x,a,b,c,f1i,x1i,f2i,x2i),

     //printf("\n x(x1i)=%f,x(x(x2i)=%f",x(x1i),x(x2i));
     
     //The case used most often is the PLQ convex hull on a bounded interval [x(1),x(5)].  
     if x(x1i) > -%inf & x(x2i) < %inf then //LOWER BOUNDRY IS FINITE, UPPER BOUNDRY IS FINITE
          y1 = a(f1i)*x(x1i)^2 + b(f1i)*x(x1i) + c(f1i);
          y2 = a(f2i)*x(x2i)^2 + b(f2i)*x(x2i) + c(f2i);
          m = (y2-y1)/(x(x2i)-x(x1i)); //slope
          i = -x(x1i)*m + y1; //intercept
          l_fctn = [x(x2i),0,m,i];
     elseif x(1) == -%inf & x(5) == %inf then //LOWER BOUNDRY IS UNBOUNDED, UPPER BOUNDRY IS UNBOUNDED
          if a(1) <> 0 & a(2) <> 0 then cerror("_plq_conv_buildl: CASE D.N.E. (1)"); //D.N.E, as x(2), & x(4) exist
          elseif a(1) <> 0 & a(2) == 0 then //special
              x(2)=-1/2*(b(1)-2*a(2)*x(3)-b(2))/a(1);
              y1 = a(1)*x(2)^2+b(1)*x(2)+c(1);
              m=2*a(2)*x(3)+b(2);
              i=-x(2)*m+y1;
              l_fctn=[x(2),a(1),b(1),c(1);x(5),0,m,i];
          elseif a(1) == 0 & a(2) <> 0 then //special
              x(4) = 1/2*(-b(2)+2*a(1)*x(3)+b(1))/a(2);
              y2 = a(2)*x(4)^2 + b(2)*x(4) + c(2);
              m=2*a(1)*x(3) + b(1);
              i=-x(4)*m + y2;
              l_fctn=[x(4),0,m,i;x(5),a(2),b(2),c(2)];
          elseif a(1) == 0 & a(2) == 0 then //D.N.E, as plqco=[%inf,0,0,-%inf]
          else cerror("_plq_conv_buildl: Case does not exist");
          end;
     elseif x(1) == -%inf & x(5) < %inf then //LOWER BOUNDRY IS UNBOUNDED, UPPER BOUNDRY IS FINITE
          if a(1) <> 0 & a(2) <> 0 then cerror("_plq_conv_buildl: CASE D.N.E (2)"); //D.N.E, as x(2), & x(4) exist
          elseif a(1) <> 0 & a(2) == 0 then cerror("_plq_conv_buildl: CASE D.N.E (3)"); //D.N.E, as x(2) exists
          elseif a(1) == 0 & a(2) <> 0 then //special
              x(4) = 1/2*(-b(2)+2*a(1)*x(3)+b(1))/a(2);
              y2 = a(2)*x(4)^2 + b(2)*x(4) + c(2);
              m=2*a(1)*x(3) + b(1);
              i=-x(4)*m + y2;
              l_fctn =[x(4),0,m,i;x(5),a(2),b(2),c(2)];
          elseif a(1) == 0 & a(2) == 0 then 
              l_fctn = [x(5),0,b(1),-x(5)*b(1)+(b(2)*x(5)+c(2))]; //special
          else cerror("_plq_conv_buildl: Case does not exist");
          end;
     elseif x(1) > -%inf & x(5) == %inf then //LOWER BOUNDRY IS FINITE, UPPER BOUNDRY IS UNBOUNDED
          if a(1) <> 0 & a(2) <> 0 then cerror("_plq_conv_buildl: CASE D.N.E. (4)"); //D.N.E, as x(2), & x(4) exist
          elseif a(1) <> 0 & a(2) == 0 then //special
              x(2) = (b(2)-b(1))/2*a(1);
              y2 = a(1)*x(2)^2 + b(1)*x(2) + c(1);
              m=b(2);
              i=-x(2)*m + y2;
              l_fctn =[x(2),a(1),b(1),c(1);x(5),0,m,i];
          elseif a(1) == 0 & a(2) <> 0 then cerror("_plq_conv_buildl: CASE D.N.E. (5)"); //D.N.E, as x(4) exists
          elseif a(1) == 0 & a(2) == 0 then //special
              l_fctn=[%inf,0,b(2),-x(1)*b(2)+(b(1)*x(1)+c(1))];
          else cerror("_plq_conv_buildl: CASE D.N.E. (6)");
          end;
     else cerror("_plq_conv_buildl: CASE D.N.E. (7)");
     end
endfunction

/////////////// ROCKAFELLAR AND FITZPATRICK FUNCTIONS ///////////////

function R = plq_rock(B, k)
  // Compute the Rockafellar PLQ function R_{A,a_k} in linear time O(m),
  // where B: x -> conv(A*x): union(i=1:m, {a(i)} cross [bminus(i), bplus(i)].
  // This same code is used in op.sci:op_fitzinf() to avoid having to
  // recalculate a, bminus, bplus across multiple plq_rock() calls.
  //
  // B = [a; bminus; bplus];    // a, bminus, bplus are row vectors.

  if (~and(B(1,2:$) - B(1,1:$-1) >= 0)) then
    [a,ak] = gsort(B(1,:), "c", "i");
    B = [a; B(2:$,ak)];
  end;

  if (or(B(1,2:$) - B(1,1:$-1) == 0)) then
    cerror("plq_rock: a values are not strictly increasing.");
    R = [];
    return;
  end;

  // Is the operator not monotonically nondecreasing?
  if (~(and(B(3,:)-B(2,:) >= 0) & and(B(2,2:$)-B(3,1:$-1) >= 0))) then
    R = [%inf, 0, 0, %inf];
    return;
  end

  m = size(B, 2);
  k = k + 1;
  a = [-%inf, B(1,:), %inf];
  bm = [%nan, B(2,:), %nan];
  bp = [%nan, B(3,:), %nan];

  R = [];
  xi = 2;
  // Precompute a sum to improve the efficiency of repeatedly
  // computing Rockafellar functions.
  Sminus = sum((a(2:k-1) - a(3:k)) .* bm(3:k)); // ..O(m)
  Splus = 0;

  for i = 2:m+1
    if (i <= k) then
      b = bm(i);
      c = -a(i)*bm(i) + Sminus;
      Sminus = Sminus - (a(i) - a(i+1))*bm(i+1);
      R = [R; a(xi), 0, b, c];
      xi = xi + 1;
    end;

    if (i >= k) then
      b = bp(i);
      c = -a(i)*bp(i) + Splus;
      Splus = Splus + (a(i+1) - a(i))*bp(i);
      R = [R; a(xi), 0, b, c];
      xi = xi + 1;
    end;
  end;
endfunction

function F = plq_fitzinf0_direct(B)
  // Compute F[A,%inf](., 0) = max[a] R[A,a](.)
  //                         = max[1<=k<=m] R[a_k](.)
  // where B: x -> conv(A*x): union(i=1:m, {a(i)} cross [bminus(i), bplus(i)].
  //
  // B = [a; bminus; bplus];    // a, bminus, bplus are row vectors.

  if (~and(B(1,2:$) - B(1,1:$-1) >= 0)) then
    [a,k] = gsort(B(1,:), "c", "i");
    B = [a; B(2:$,k)];
  end;

  if (or(B(1,2:$) - B(1,1:$-1) == 0)) then
    cerror("op_fitzinf0_direct: a values are not strictly increasing.");
    F = [];
    return;
  end;

  // Is the operator not monotonically nondecreasing?
  if (~(and(B(3,:)-B(2,:) >= 0) & and(B(2,2:$)-B(3,1:$-1) >= 0))) then
    F = [%inf, 0, 0, %inf];
    return;
  end;

  F = [%inf, 0, 0, -%inf];
  F = plq_rock(B, 1);
  for k = 2:size(B, 2)
    R = plq_rock(B, k);   // Linear.
    F = plq_max(F, R);    // Linear.  plq_max (plq_oploop actually) cleans F.
  end;
endfunction

function plqf = plq_fitzinf0(B)
  // Compute F[A,%inf](., 0) = max[a] R[A,a](.)
  //                         = max[1<=k<=m] R[a_k](.)
  // where B: x -> conv(A*x): union(i=1:m, {a(i)} cross [bminus(i), bplus(i)].
  //
  // B = [a; bminus; bplus];    // a, bminus, bplus are row vectors.
  //
  // This is an optimized version of plq_fitzinf0_direct,
  // which runs in big-theta(m) time.

  if (~and(B(1,2:$) - B(1,1:$-1) >= 0)) then
    [a,k] = gsort(B(1,:), "c", "i");
    B = [a; B(2:$,k)];
  end;

  if (or(B(1,2:$) - B(1,1:$-1) == 0)) then
    cerror("op_fitzinf0: a values are not strictly increasing.");
    plqf = [];
    return;
  end;

  // Is the operator not monotonically nondecreasing?
  if (~(and(B(3,:)-B(2,:) >= 0) & and(B(2,2:$)-B(3,1:$-1) >= 0))) then
    plqf = [%inf, 0, 0, %inf];
    return;
  end;

  m = size(B, 2);
  a = [-%inf; B(1,:)'; %inf];
  bm = [-%inf; B(2,:)'; %inf];
  bp = [-%inf; B(3,:)'; %inf];

  // Todo: Use binary searches.
  j0 = max(find(bm < 0));
  j1 = max(find(bp < 0));

  S1 = sum((a(2:j0-1) - a(3:j0)) .* bm(3:j0));
  S2 = 0;

  // Compute F[A,%inf].  O(m)
  plqf = [];
  for i = 1:m+1
    plqx = a(i+1);    // a(i) < x < a(i+1)

    if (j0 > i) then
      // Case 2: R_{a_{j_0}}
      //printf("plq_fitzinf0, i = %d, Case 2\n", i);
      plqb = bm(i+1);
      plqc = -a(i+1)*bm(i+1) + S1;
      S1 = S1 - (a(i+1) - a(i+2)) * bm(i+2);
    else
      if (i-1 >= j1+1) then
        S2 = S2 + (a(i) - a(i-1)) * bp(i-1);
        plqc = S2;
        //printf("Sum %f, ", S2);
      else
        plqc = 0;
        //printf("No sum %f, ", S2);
      end;

      if (j0 < i) then
        // Case 1: R_{a_{j_1 + 1}}
        //printf("plq_fitzinf0, i = %d, Case 1\n", i);
        plqb = bp(i);
        plqc = plqc - a(i)*bp(i);
      elseif (bp(i) == bm(i+1) | i == m+1) then
        // Case 3: R_{a_{j_1 + 1}}
        //printf("plq_fitzinf0, i = %d, Case 3\n", i);
        plqb = bp(i);
        plqc = plqc - a(i)*bp(i);
      else
        // Case 4: R_{a_{j_1 + 1}} then R_{a_{i+1}}
        //printf("plq_fitzinf0, i = %d, Case 4\n", i);
        if (i ~= 1) then
          xbar = (a(i)*bp(i) - a(i+1)*bm(i+1)) / (bp(i) - bm(i+1));
          plqb = bp(i);
          plqc = plqc - a(i)*bp(i);
          plqf = [plqf; xbar, 0, plqb, plqc];
        end;

        plqb = bm(i+1);
        plqc = -a(i+1)*bm(i+1);
      end;
    end;

    plqf = [plqf; plqx, 0, plqb, plqc];
  end;

  plqf = plq_clean(plqf);
endfunction
