mode(-1);

clear;
global CCADIR;   // Bring back CCADIR if the user was using it.

function [name] = getMonth(month)
	select month
		case 1 then name = "January";
		case 2 then name = "February";
		case 3 then name = "March";
		case 4 then name = "April";
		case 5 then name = "May";
		case 6 then name = "June";
		case 7 then name = "July";
		case 8 then name = "August";
		case 9 then name = "September";
		case 10 then name = "October";
		case 11 then name = "November";
		case 12 then name = "December";
	end
endfunction



// user set globals
global __test_file;
global __test_exclude;

// automated globals
global __test_set;
global __test_failures;
global __test_passes;
global __base_dir;
global __test_dir;
global __test_coverage;


// define base directory
__cwd_dir = pwd();
__test_dir = get_absolute_file_path('test.sci');
chdir(__test_dir+"../");
__base_dir = pwd();
chdir(__test_dir);

// Load the coverage testing library
[fp, err] = mopen('coverage.sci', 'r');
if (err ~= 0) then
  __test_coverage = %f;
else
  __test_coverage = %t;
  mclose(fp);
  exec('coverage.sci');
end;

//Number of functions and libraries loaded
// chdir(__base_dir+"/c_routines/");
// exec('dt_c.sce');
chdir(__base_dir);
CCA_lib = ['common.sci','me.sci','lft.sci','plq.sci','op.sci']';

if (__test_coverage) then
  cover_reset();
  [filesProfiled, success, reason] = cover_profileFiles(CCA_lib);
  if (~success) then
    __test_coverage = %f;
  else
    if (size(filesProfiled, 1) ~= 0) then
      printf("\nNeeded to reprofile ");
      cont = "";
      for filename = filesProfiled'
        printf("%s%s", cont, filename);
        cont = ", ";
      end;
      printf(".");
    end;
    clear cover_loadedFile; getf(cover_profiledName('common.sci'), 'p'); cover_newFile();
    clear cover_loadedFile; getf(cover_profiledName('me.sci'), 'p'); cover_newFile();
    clear cover_loadedFile; getf(cover_profiledName('lft.sci'), 'p'); cover_newFile();
    clear cover_loadedFile; getf(cover_profiledName('plq.sci'), 'p'); cover_newFile();
    clear cover_loadedFile; getf(cover_profiledName('op.sci'), 'p'); cover_newFile();
    Nb_fcts_CCA = size(cover_loadedFns(), 1);
  end;
end;

if (~__test_coverage) then
  Nb_fcts_scilab = size(who('local'),1);
  x=1; while x <= size(CCA_lib,1),exec (CCA_lib(x)); x = x + 1;,end;
  Nb_fcts_CCA = size(who('local'),1) - Nb_fcts_scilab
end;

chdir(__test_dir);

// this must be defined at the unit test level
__test_set  = "N/A";
__test_failures = 0;
__test_passes = 0;

// set up the log file
__test_filename = pwd()+'/test.log';//xgetfile("*", "/home",'Please select the location for the log file.');
__test_output = mopen(__test_filename,"w");

// write the header
__test_date = getdate();
fprintf(__test_output, "Scilab Unit Test Log :: %s %d, %d\n", getMonth(__test_date(2)), __test_date(6), __test_date(1));
fprintf(__test_output, "==============================================================================\n");
fprintf(__test_output, "Set Name :: Test Name ................................................[Result]\n");
fprintf(__test_output, "------------------------------------------------------------------------------\n");

// extremely useful for checking if a test failed
// the first call to this must be bool = checkForFail(check, %t), and then following calls
// must then be bool = checkForFail(check, bool)
function [b] = checkForFail(bool, lastBool)
	if (bool == %f | lastBool == %f)
		b = %f;
	else
		b = %t;
	end
endfunction

// In each unit test file's runTestFile() function,
// if b is the boolean representing the success of
// the function, testWrapper should be called like:
// b = checkForFail(testWrapper(testFoo, "testFoo"), b);
// The test succeeds if testFoo returns logical true,
// fails if testFoo returns logical false or errors.
function [b] = testWrapper(testFunc, __test_name)
	global __test_failures;
	global __test_passes;
	r = %f;
	
	errcatch(-1,"continue");  // catch all errors
		r = testFunc();
	if (iserror(-1))
		status = "[ERROR!]"
		__test_failures = __test_failures + 1
		b = %f
	elseif (r)
		status = "[Passed]"
		__test_passes = __test_passes + 1
		b = %t
	else
		status = "[FAILED]"
		__test_failures = __test_failures + 1
		b = %f
	end
	errclear(-1);
	
	line = msprintf("%s :: %s ", __test_set, __test_name);
	while (length(line) < 70), line = msprintf("%s.",line); end;
	
	mfprintf(__test_output, "%s%s\n", line,status);
endfunction

// This function acts like testWrapper, except the
// test succeeds if the function errors, and fails
// if the test returns without erroring.  It is used
// to test for error cases of functions.
function [b] = errorTestWrapper(testFunc, __test_name)
  global __test_failures;
  global __test_passes;
  global __testing_errors;

  __testing_errors = 10001;
  errcatch(-1, "continue", "nomessage");
  testFunc();
  b = iserror(10001);
  __testing_errors = %f;

  if (b) then
    status = "[Passed]";
    __test_passes = __test_passes + 1;
  elseif (iserror(-1)) then
	status = "[ERROR!]";
	__test_failures = __test_failures + 1;
  else
    status = "[FAILED]";
    __test_failures = __test_failures + 1;
  end;

  errclear(-1);

  line = msprintf("%s :: %s ", __test_set, __test_name);
  while (length(line) < 70), line = msprintf("%s.",line); end;

  mfprintf(__test_output, "%s%s\n", line,status);
endfunction

// This function works like errorTestWrapper, but
// when an error occurs, Scilab pauses instead of
// continuing.  This can be used to debug an error
// condition when Scilab freezes instead of returning
// an error cleanly in an error test.
function [b] = breakTestWrapper(testFunc, __test_name)
  global __test_failures;
  global __test_passes;
  global __testing_errors;

  __testing_errors = 10001;
  errcatch(-1, "pause");
  testFunc();
  b = iserror(10001);
  __testing_errors = %f;

  if (b) then
    status = "[Passed]";
    __test_passes = __test_passes + 1;
  elseif (iserror(-1)) then
	status = "[ERROR!]";
	__test_failures = __test_failures + 1;
  else
    status = "[FAILED]";
    __test_failures = __test_failures + 1;
  end;

  errclear(-1);

  line = msprintf("%s :: %s ", __test_set, __test_name);
  while (length(line) < 70), line = msprintf("%s.",line); end;

  mfprintf(__test_output, "%s%s\n", line,status);
endfunction

// run tests - note that these script files will run their tests through function 
// testWrapper already
function [b,testFiles] = runTests()
	// if the user defined __test_file, we'll just test that file
	global __test_file;
	global __test_exclude;
	
	if (length(__test_file) == 0)
		if (MSDOS)
			fileList=ls("test_*.sc?");
		else
			fileList=ls("test_*.sc?","-1");
		end;
	else
		fileList=__test_file;
	end
	
	b=%t;
  testFiles = size(fileList,1);
	for curFileNum=1:size(fileList,1)
		curFile=fileList(curFileNum)
		curFile=stripblanks(curFile);
		
		// if not excluded, execute test files
		if (length(find(__test_exclude == curFile)) == 0)
			clear runTestFile;
			exec(curFile,-1);
			results = runTestFile();
			b = checkForFail(results, b);
		
			if (curFileNum < size(fileList, 1))
				mfprintf(__test_output, "------------------------------------------------------------------------------\n");
			end
		end
	end

	mfprintf(__test_output, "==============================================================================\n");
	if (b)
		mfprintf(__test_output, "All tests passed!\n");
	else
		mfprintf(__test_output, "%d test(s) failed!\n", __test_failures);
	end 
endfunction

//printf("Automated Test Suite\n");
//printf("==================================================================\n");

// Print out all of the global variable settings pertaining to tests
//printf("\nGlobal Variable Settings\n");
//printf("------------------------\n");
if (length(__test_file) > 0)
	//printf("__test_file:     %s\n", __test_file);
else
	//printf("__test_file:     All files starting with test\n");
end
if (size(__test_exclude, 1) > 0)
	//printf("__exclude_file:  %s\n", __test_exclude(1));
	if (size(__test_exclude, 1) > 1)
		for i=2:size(__test_exclude, 1)
			//printf("                 %s\n", __test_exclude(i));
		end
	end
else
	//printf("__exclude_file: No files excluded\n");
end
//printf("------------------------\n");

answer = 1;//x_choose(['Yes';'No'],['Should I run the tests';'with these settings?']);

if (answer(1) == 1)
	// run all tests!
    printf("\nRunning tests...");
	[__test_results,testFiles] = runTests();
    if (__test_coverage) then
      printf("\nCollecting coverage info...");
      cov = cover_results();
      mfprintf(__test_output, "\n");
      cover_printStmtCoverage(cov, __test_output);
    end;

	// give final results
	if (__test_results)
		mprintf("\nAll tests passed!");//x_message("All tests passed!");
	else
//		if (__test_failures > 1)
//			smessage = sprintf("%d tests failed, check log file: %s", __test_failures, __test_filename);
//		else
//			smessage = sprintf("%d test failed, check log file: %s", __test_failures, __test_filename);
//		end
//		printf(smessage);//x_message(smesnsage);
		mprintf("\n%d test(s) failed, check log file: %s", __test_failures, __test_filename);
	end
	
	mclose(__test_output)// close file
else
	mprintf("Tests not ran\n");
end

//displaying detail unit test information
__test_total = __test_passes + __test_failures;

mprintf("\n------------------------");
mprintf("\n %d/%d unit tests passed/run",__test_passes,__test_total);
mprintf("\n %d functions loaded/run",Nb_fcts_CCA);
mprintf("\n %d libraries loaded",size(CCA_lib,1))
mprintf("\n %d files run",testFiles);
if (__test_coverage) then
  mprintf("\n %d%% statement coverage", cov.stmtCoverage(3)*100);
  fnsMissed = cov.fnCoverage(2) - cov.fnCoverage(1);
  if (fnsMissed > 0) then
    mprintf(" (%d functions not tested, see log file)", fnsMissed);
  end;
else
  mprintf("\n Coverage testing library not found.");
end;
mprintf("\n------------------------\n");

//whos -type -function

chdir(__cwd_dir);//restore current directory
