// Original Author: Yves Lucet
// Author:          Jeff Dicker
// Whatsit:         Unit tests for lft_direct, lft_direct_d, lft_llt, lft_llt_d
// Date:            Summer 2005

mode(-1);
__test_set = "LLT: Linear Time Legendre Transform";

function y = f(x), y = x .^ 2 / 2; endfunction
function s = df(x), s = x; endfunction

function b = testPartialConj_direct()
//compute the conjugate of a function going through (i,row(i))
//with lft_direct_d
	row = [1 2.5 5 8.5 13]';//values of the function on {1,2,3,4,5}
	conj_part = lft_direct_d(row);//conjugate computation
	b = isequal(conj_part, [0 1.5 4 7.5 12]');//verify the result
endfunction

function b = testQuadratic_direct_plt()
//conjugate of the energy function f(x)=x^2/2
//with lft_direct
	X = [1:1:10]';//primal grid
	Y = 0.5.*(X.^2);//sample the function on X
	//compute the conjugate with dual grid = primal grid
	[Conj] = lft_direct(X, Y, X);
	[Conjp] = lft_plt(X, f, df);
	// Check that the conjugate of the energy function is itself
	b = isequal(Y, Conj, Conjp);
endfunction

function b = testPartialConj_llt()
//compute the conjugate of a function going through (i,row(i))
//with lft_llt
	row = [1 2.5 5 8.5 13]';//values of the function on {1,2,3,4,5}
	conj_part = lft_llt_d(row);//conjugate computation
	b = isequal(conj_part, [0 1.5 4 7.5 12]');//verify the result
endfunction

function b = testQuadratic_llt()
//conjugate of the energy function f(x)=x^2/2
//with lft_llt
	X = [1:1:10]';//primal grid
	Y = 0.5.*(X.^2);//sample the function on X
	//compute the conjugate with dual grid = primal grid
	[Conj] = lft_llt(X, Y, X, %t, 0);
	// Check that the conjugate of the energy function is itself
	b = isequal(Y, Conj);
endfunction

function b = testCompareDirLLTFusion()
//compare lft_llt with option fusion,  lft_llt with option fusionsci, 
//and lft_direct when computing the conjugate of discrete data 
	X = [1:1:10]';//primal grid
	Y = [34 3 1 15 13 12 22 30 32 58]';//values of an ungiven function on X
	ConjA = lft_llt(X, Y, X, %f);//result with llt algorithm option fusionsci
	ConjAa= lft_llt(X, Y, X, %f, 0);//result with llt algorithm option fusion
	ConjB = lft_direct(X, Y, X);//result with direct algorithm (quadratic)
	ConjC = cca_fenchel(X, Y, X, %f);
	b = isequal(ConjA, ConjAa, ConjB, ConjC);//verify the result
endfunction

function b = testLft2dEmptyS()
	X = (1:4)';
	Y = (1:4)';
	f = X * Y';
	ConjA1 = lft_direct2d(X, Y, f, [], []);
	ConjA2 = lft_llt2d   (X, Y, f, [], [], %t);
	ConjB1 = lft_direct2d(X, Y, f,  0, []);
	ConjB2 = lft_llt2d   (X, Y, f,  0, [], %t);
	ConjC1 = lft_direct2d(X, Y, f, [],  0);
	ConjC2 = lft_llt2d   (X, Y, f, [],  0, %t);
	ConjD  = cca_fenchel (X, Y, f, [], [], %t);
	b = isEqual([], ConjA1, ConjA2, ConjB1, ConjB2, ConjC1, ConjC2, ConjD);
endfunction

function testLftLlt2dWrongSize()
  Xr = 1:4; Xc = 1:5; f = eye(5, 5);
  Sr = 1:3; Sc = 2:5;
  lft_llt2d(Xr, Xc, f, Sr, Sc);
endfunction

//runs all the above unit test functions
function b = runTestFile()
	b = %t;
	b = checkForFail(testWrapper(testPartialConj_direct, "testPartialConj_direct"), b);
	b = checkForFail(testWrapper(testQuadratic_direct_plt, "testQuadratic_direct_plt"), b);
	b = checkForFail(testWrapper(testPartialConj_llt, "testPartialConj_llt"), b);
	b = checkForFail(testWrapper(testQuadratic_llt, "testQuadratic_llt"), b);
	b = checkForFail(testWrapper(testCompareDirLLTFusion, "testCompareDirLLTFusion"), b);
	b = checkForFail(testWrapper(testLft2dEmptyS, "testLft2dEmptyS"), b);
	b = checkForFail(errorTestWrapper(testLftLlt2dWrongSize, "testLftLlt2dWrongSize"), b);
endfunction
