// Author:   Mike Trienis
// For:      Dr. Yves Lucet
// Whatsit:  A test file for plq proximal average

// I = Indicator, E = Exponential, C = Conjugate, L = Linear, Q = Quadratic

mode(-1);
__test_set = "PLQ: plq_pa - Proximal average of two plq functions";

function [b] = testL2Lpa()
	b = %f;
	lambda_pa = 0.5;
	f1 = [%inf, 0, 1, 0];
	f2 = [%inf, 0, -1, 5]; 
	desired = [%inf,0,0,2];
	result = plq_pa(f1,f2,lambda_pa),
	result2 = plq_pa_mu(1,lambda_pa,f1,f2),
	b = isEqual(desired, result, result2),
endfunction

function [b] = testI2Ipa()
	b = %f;
	lambda_pa = 0.5;
	f2 = [5,0,0,1];
	f1 = [6,0,0,4];
	desired = [5.5,0,0,2.625];
	result = plq_pa(f1,f2,lambda_pa),
	result2 = plq_pa_mu(1,lambda_pa,f1,f2),
	b = isEqual(desired, result, result2),
endfunction

function [b] = testE2Cpa()
	b = %f;
	lambda_pa = 1;
	f0 = [0,0,0,0];
	f1 = [0,0,0,0];

	desired = [5.5,0,0,2.625];
	result = plq_pa(f1,f2,lambda_pa),
	result2 = plq_pa_mu(1,lambda_pa,f1,f2),
	b = isEqual(desired, result, result2),
endfunction

function [b] = testL2Qnonconvex()
  b = %f;
  f1 = [-1,0,-1,-2;1,0,1,0;%inf,0,-1,2];
  f2 = [0,-1,0,0;%inf,1,0,0];
  plq_plotpa_mu(f1,f2,-2:0.25:2,0:0.1:1,1,[-2,-5,3,10]); plq_plot2(-2,3,f1,f2);
  lambda = 0.5;
  mu = 0.5;
  plq
endfunction

function [b] = runTestFile()
	b = %t;
	b = checkForFail(testWrapper(testL2Lpa, "testL2Lpa"), b);
	b = checkForFail(testWrapper(testI2Ipa, "testI2Ipa"), b);
	//b = checkForFail(testWrapper(testE2Cpa, "testE2Cpa"), b);
endfunction
